#!/usr/bin/env python3

"""uctenkovka.py - part of uctenkovkapy project
Copyright (c) 2019 Jakub Smetana <jakub.smetana@protonmail.com>
Licenced under GPL3.0-or-later - see LICENSE file.
Freely availiable at <https://gitlab.com/jsmetana/uctenkovkapy>
Please report BUGS and anything else at:
<https://gitlab.com/jsmetana/uctenkovkapy/issues>"""

import os
import time
import json
import clius
import uctenkovkapy

if __name__ == '__main__':
    whole_time = time.monotonic()

    log = clius.logger.Logger(None, 7)

    log.info(__doc__)

    if len(clius.sys.argv) < 2 or len(clius.sys.argv) > 3:
        log.alert('Argument count is not 1!')
        log.notice('Usage: uctenkovka.py <FILE> [-m|--multiple-accounts]')
        log.critical('Exiting due to an error!')
        exit(1)

    multiple = False

    if len(clius.sys.argv) == 3:
        if clius.sys.argv[2] == '-m' or clius.sys.argv[2] == '--multiple-accounts':
            log.info('Running with multiple-accounts...')
            multiple = True

    file = clius.sys.argv[1]

    log.notice('Checking for file "{}"...', file)

    if not os.path.isfile(file):
        log.alert("File {} doesn't exist!", file)
        log.critical('Exiting due to an error!')
        exit(2)

    log.warning('File {} found!', file)
    log.notice('Loading file {} in json format...', file)

    with open(file) as f:
        content = json.load(f)

    log.warning('Loaded file {}!', file)
    log.notice('Loading receipt(s)...')

    prepared_upload = []
    if type(content) is list:
        if multiple:
            upload = [[]]
            # for receipt in content:
            #     if receipt['date'] not in receipts.keys():
            #         receipts[receipt['date']] = [receipt]
            #     else:
            #         receipts[receipt['date']].append(receipt)
            # for _, val in receipts:
            for receipt in content:
                for account in upload:
                    for account_receipt in account:
                        if receipt['multi'] == account_receipt['multi'] and \
                                receipt['date'] == account_receipt['date']:
                            break
                    else:
                        continue
                    account.append(receipt)
                    break
                else:
                    continue
                upload.append([receipt])
        else:
            upload = content
        log.warning('Loaded [{}] receipts!', len(content))
        if len(content) == 0:
            log.alert('Found 0 (ZERO) receipts! Will not continue...')
            log.critical('Exiting due to an error!')
            exit(3)
        if multiple:
            for u in range(len(upload)):
                log.warning('Found [{}] receipts for [{}] account!'
                            .format(len(upload[u]), u))
            if len(upload) != 1:
                to_upload = log.question('Select which accounts to upload'
                                         ' [use + to separate or * for all]:\n')
                if to_upload == '':
                    log.alert('Abandoned!')
                    log.warning('Done! Exiting')
                    exit(0)
                elif to_upload == '*':
                    prepared_upload = upload
                else:
                    to_upload = to_upload.split('+')
                    for t in to_upload:
                        try:
                            t = int(t)
                        except ValueError:
                            log.alert('Value "{}" can not be cast to an integer'
                                      ' - skipping...')
                        prepared_upload.append(upload[t])
        else:
            prepared_upload = [upload]
        if len(prepared_upload) == 0:
            log.alert('Prepared to upload is EMPTY!')
            log.critical('Exiting due to an error!')
            exit(3)
    else:
        log.alert("JSON doesn't have a list in the root!")
        log.critical('Exiting due to an error!')
        exit(4)

    sessions = {}

    for account in range(len(prepared_upload)):
        while 1:
            log.notice('Getting credentials for {}. account.', account)
            token = log.question('Token for account [leave empty for normal login]: ')
            if token == '':
                username = log.question('Username [e-mail]: ')
                password = log.getpass('Password [hides input fully]: ')
                try:
                    log.notice('Logging in...')
                    sessions[uctenkovkapy.Session(username, password)]\
                        = prepared_upload[account]
                    log.warning('Successfully logged in!')
                    break
                except ConnectionRefusedError:
                    log.critical('Error with logging in, probably bad '
                                 'credentials... Try again:')
            else:
                sessions[uctenkovkapy.Session(None, None, token=token)]\
                    = prepared_upload[account]
                break

    all_count = 0
    for session, receipts in sessions.items():
        all_count += len(receipts)

    log.notice('Starting upload...')

    for session, receipts in sessions.items():
        log.notice('Starting upload for account {}/{}.',
                   list(sessions.keys()).index(session) + 1,
                   len(sessions))
        for receipt in receipts:
            log.debug('Uploading {}...', receipt['code'])
            try:
                receipt.pop('multi')
            except KeyError:
                pass
            session.register(**receipt)
            log.warning('Uploaded {}/{}/{}',
                        receipts.index(receipt) + 1,
                        len(receipts),
                        all_count)

    log.warning('Done! in {}s', time.monotonic() - whole_time)
